﻿using System;
using System.Data.Entity;
using System.Diagnostics;
using System.Runtime.Remoting.Contexts;
using DomainClasses;

namespace DataModel {
  public class MonkeysContext : DbContext {
    public DbSet<Monkey> Monkeys { get; set; }
    public DbSet<Banana> Bananas { get; set; }
    public DbSet<Country> Countries { get; set; }

    public override int SaveChanges() {
      foreach (var entry in ChangeTracker.Entries<IObjectWithState>())
{
        IObjectWithState stateInfo = entry.Entity;
        switch (stateInfo.State) {
          case LocalState.Added:
            entry.State = EntityState.Added;
            break;
          case LocalState.Unchanged:
            entry.State = EntityState.Unchanged;
            break;
          case LocalState.Modified:
            entry.State = EntityState.Modified;
            break;
          case LocalState.Deleted:
            entry.State=EntityState.Deleted;
            break;
          default:
            throw new ArgumentOutOfRangeException();
        }
      }
      foreach (var entry in ChangeTracker.Entries<Monkey>()) {
        Debug.WriteLine("Monkey {0} State is {1}",
          (entry.Entity).Name,entry.State);
        
      }
      foreach (var entry in ChangeTracker.Entries<Banana>()) {
        Debug.WriteLine("Banana {0} eaten by Monkey Id {1} State is {2}",
          (entry.Entity).SerialNumber,(entry.Entity).MonkeyId, entry.State);

      }
      return 0;
      // return base.SaveChanges();
    }
  }

  public class MyInitAndSeed : DropCreateDatabaseIfModelChanges<MonkeysContext> {
    protected override void Seed(MonkeysContext context) {
      context.Countries.Add(new Country {Name = "US"});
      context.Countries.Add(new Country {Name = "United Kingdom"});

      context.Countries.Add(new Country {Name = "Uruguay"});
      context.Countries.Add(new Country {Name = "Uganda"});
      base.Seed(context);
    }
  }
}