﻿using System.Collections.Generic;
using System.Runtime.CompilerServices;

[assembly: InternalsVisibleTo("Tests")]

namespace DomainClasses {
  public class Monkey : IObjectWithState {
    public Monkey() {
      State = LocalState.Unchanged;
      Bananas = new List<Banana>();
    }

    private Monkey(string name, int countryOfOriginId) {
      Bananas = new List<Banana>();
      Name = name;
      CountryOfOriginId = countryOfOriginId;
      State = LocalState.Added;
    }


    public int Id { get; private set; }
    public string Name { get; private set; }
    public List<Banana> Bananas { get; set; }
    public Country CountryOfOrigin { get; set; }
    public int CountryOfOriginId { get; set; }
    public LocalState State { get; set; }

    public static Monkey Create(string name, int countryOfOriginId) {
      return new Monkey(name, countryOfOriginId);
    }

    public void Rename(string name) {
      Name = name;
      if (State == LocalState.Unchanged) {
        State = LocalState.Modified;
      }
    }

    public void AteABanana(string serialNumber) {
      Bananas.Add(Banana.Create(serialNumber, Id));
    }
  }

  public class Banana : IObjectWithState {
    public Banana() {
      State = LocalState.Unchanged;
    }

    private Banana(string serialNumber, int monkeyId) {
      SerialNumber = serialNumber;
      MonkeyId = monkeyId;
      State = LocalState.Added;
    }

    public int Id { get; private set; }
    public string SerialNumber { get; private set; }
    public int MonkeyId { get; private set; }
    public Monkey Monkey { get; set; }
    public LocalState State { get; private set; }

    public static Banana Create(string serialNumber, int monkeyId) {
      return new Banana(serialNumber, monkeyId);
    }
  }

  public class Country {
    public int Id { get; set; }
    public string Name { get; set; }
  }
}